<?php
/*
Plugin Name: Tweet Posts
Plugin URI: http://joshbetz.com/2012/08/tweet-posts/
Description: Tweet your posts.
Version: 0.1
Author: Josh Betz
Author URI: http://joshbetz.com
*/

require_once( 'twitteroauth.php' );

class Tweet_Posts {

	const VERSION = '0.1';
	const SLUG    = 'tweet-posts';
	const OPTION  = 'tweet_posts';

	private $keys = array();
	
	function __construct() {
		add_action( 'init', array( $this, 'init' ) );
		add_action( 'admin_init', array( $this, 'admin_init' ) );
		add_action( 'admin_menu', array( $this, 'admin_menu' ) );
		add_action( 'publish_post', array( $this, 'publish_post' ) );
		
		$this->keys = get_option( self::OPTION );
	}

	function init() {
		register_deactivation_hook( __FILE__, array( $this, 'deactivate' ) );
	}

	function deactivate() {
		unregister_setting( self::OPTION, self::OPTION, array( $this, 'validate_settings' ) );
	}

	function publish_post( $id ) {
		$post = get_post( $id );
		$title = $post->post_title;
		$shortlink = wp_get_shortlink( $id );
		$type = get_post_format( $id );
		
		$tweet = self::get_tweet( $title, $shortlink, $type );
		$this->do_tweet( $tweet );
	}

	function do_tweet( $tweet ) {
		$connection = new TwitterOAuth( $this->keys['consumer_key'], $this->keys['consumer_secret'], $this->keys['oauth_token'], $this->keys['oauth_token_secret'] );

		$args = array(
			'status' => esc_html( $tweet ),
			'include_entities' => true
		);
		$response = $connection->post( 'statuses/update', $args );

		return $response;
	}

	static function get_tweet( $_title, $shortlink, $type ) {
		switch ( $type ) {
			case 'aside':
			case 'status':
			case 'quote':
			case 'chat':
				$pre = "New $type: ";
				break;

			case 'audio':
			case 'video':
			case 'image':
			case 'link':
				$pre = "New $type post: ";
				break;

			case 'gallery':
				$pre = "New image gallery: ";
				break;

			case 'standard':
			default:
				$pre = "New blog post: ";
		}

		if ( empty( $_title ) )
			return $pre . $shortlink;

		$left = 140 - strlen( $pre . $shortlink );
		if ( strlen( $_title ) < $left )
			$title = $_title . ' ';
		else {
			$_title = substr( $_title, 0, $left - 4 );
			$parts = explode( ' ', $_title );
			array_pop( $parts );
			$title = implode( ' ', $parts ) . '... ';
		}
		
		return $pre . $title . $shortlink;
	}

	function admin_menu() {
		add_options_page( __( 'Tweet Posts', 'tweet-posts' ), __( 'Tweet Posts', 'tweet-posts' ), 'manage_options', self::SLUG, array( $this, 'settings_page' ) );
	}

	function settings_page() { ?>
		<div class="wrap">
			<h2><?php _e( 'Tweet Posts', 'tweet-posts' ); ?></h2>
			<form action="options.php" method="post">
				<?php settings_fields( self::OPTION ); ?>
				<?php do_settings_sections( self::SLUG ); ?>
				<?php submit_button(); ?>
			</form>
		</div>
<?php }

	function admin_init() {
		register_setting( self::OPTION, self::OPTION, array( $this, 'validate_settings' ) );
		add_settings_section( self::SLUG, __( 'Twitter API', 'tweet-posts' ), array( $this, 'api_section' ), self::SLUG );
		add_settings_field( 'consumer_key', __( 'Consumer Key', 'tweet-posts' ), array( $this, 'input' ), self::SLUG, self::SLUG, array( 'name' => 'consumer_key' ) );
		add_settings_field( 'consumer_secret', __( 'Consumer Secret', 'tweet-posts' ), array( $this, 'input' ), self::SLUG, self::SLUG, array( 'name' => 'consumer_secret' ) );
		add_settings_field( 'oauth_token', __( 'OAuth Token', 'tweet-posts' ), array( $this, 'input' ), self::SLUG, self::SLUG, array( 'name' => 'oauth_token' ) );
		add_settings_field( 'oauth_token_secret', __( 'OAuth Token Secret', 'tweet-posts' ), array( $this, 'input' ), self::SLUG, self::SLUG, array( 'name' => 'oauth_token_secret' ) );
	}

	function api_section() { ?>
		<p>Fill in your Twitter API details.</p>
<?php }

	function input( $post ) {
		$options = $this->keys;
		$setting = $post['name'];
		$value = !empty( $options ) ? $options[$setting] : ''; ?>
		<input id='<?php echo $setting; ?>' name='<?php echo self::OPTION . '[' . $setting . ']'; ?>' size='40' type='text' value='<?php echo $value; ?>' />
<?php }

	function validate_settings( $input ) {
		$new['consumer_key'] = esc_attr( $input['consumer_key'] );
		$new['consumer_secret'] = esc_attr( $input['consumer_secret'] );
		$new['oauth_token'] = esc_attr( $input['oauth_token'] );
		$new['oauth_token_secret'] = esc_attr( $input['oauth_token_secret'] );
		return $new;
	}
	
}

new Tweet_Posts();